import {
  Controller,
  Get,
  HttpStatus,
  Param,
  Query,
  UseInterceptors,
  UsePipes,
  ValidationPipe,
} from '@nestjs/common';
import { ApiOperation, ApiResponse, ApiTags } from '@nestjs/swagger';
import { MultitenancyParams, ResponseFormatInterceptor } from '@ocm/shared';

import { CredentialOffersService } from './credential-offers.service.js';
import { GetByIdParams } from './dto/get-by-id.dto.js';

@Controller()
@UsePipes(new ValidationPipe({ transform: true, whitelist: true }))
@UseInterceptors(ResponseFormatInterceptor)
@ApiTags('Credential Offers')
export class CredentialOffersController {
  public constructor(private readonly service: CredentialOffersService) {}

  @Get()
  @ApiOperation({
    summary: 'Fetch a list of credential offers',
    description:
      'This call provides a list of credential offers for a given tenant',
  })
  @ApiResponse({
    status: HttpStatus.OK,
    description: 'Credential offers fetched successfully',
    content: {
      'application/json': {
        schema: {},
        examples: {
          'Credential offers fetched successfully': {
            value: {
              statusCode: 200,
              message: 'Credential offers fetched successfully',
              data: [
                {
                  id: '71b784a3',
                },
              ],
            },
          },
        },
      },
    },
  })
  @ApiResponse({
    status: HttpStatus.NOT_FOUND,
    content: {
      'application/json': {
        schema: {},
        examples: {
          'Credential offer not found': {
            value: {
              statusCode: 404,
              message: 'Credential offer not found',
              data: null,
            },
          },
          'Tenant not found': {
            value: {
              statusCode: 404,
              message: 'Tenant not found',
              data: null,
            },
          },
        },
      },
    },
  })
  @ApiResponse({
    status: HttpStatus.INTERNAL_SERVER_ERROR,
    description: 'Something went wrong',
    content: {
      'application/json': {
        schema: {},
        examples: {
          'Something went wrong': {
            value: {
              statusCode: 500,
              message: 'Something went wrong',
              error: 'Internal Server Error',
            },
          },
        },
      },
    },
  })
  public find(@Query() { tenantId }: MultitenancyParams) {
    return this.service.findCredentialOffers(tenantId);
  }

  @Get(':credentialOfferId')
  @ApiOperation({
    summary: 'Fetch a credential offer by ID',
    description: 'This call provides a credential offer for a given ID',
  })
  @ApiResponse({
    status: HttpStatus.OK,
    description: 'Credential offer fetched successfully',
    content: {
      'application/json': {
        schema: {},
        examples: {
          'Credential offer fetched successfully': {
            value: {
              statusCode: 200,
              message: 'Credential offer fetched successfully',
              data: {
                id: '71b784a3',
              },
            },
          },
        },
      },
    },
  })
  @ApiResponse({
    status: HttpStatus.NOT_FOUND,
    content: {
      'application/json': {
        schema: {},
        examples: {
          'Credential offer not found': {
            value: {
              statusCode: 404,
              message: 'Credential offer not found',
              data: null,
            },
          },
          'Tenant not found': {
            value: {
              statusCode: 404,
              message: 'Tenant not found',
              data: null,
            },
          },
        },
      },
    },
  })
  @ApiResponse({
    status: HttpStatus.INTERNAL_SERVER_ERROR,
    description: 'Something went wrong',
    content: {
      'application/json': {
        schema: {},
        examples: {
          'Something went wrong': {
            value: {
              statusCode: 500,
              message: 'Something went wrong',
              error: 'Internal Server Error',
            },
          },
        },
      },
    },
  })
  public getById(
    @Param() { credentialOfferId }: GetByIdParams,
    @Query() { tenantId }: MultitenancyParams,
  ) {
    return this.service.getCredentialOfferById(tenantId, credentialOfferId);
  }
}
